﻿////////////////////////////////////////////////////////////////////////////////////////////////////
//
// VSCoverageToEmma - Visual Studio Coverage file to Emma converter
// Copyright (c) Kouji Matsui, All rights reserved.
//
// * Redistribution and use in source and binary forms, with or without modification,
//   are permitted provided that the following conditions are met:
// * Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
// * Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
////////////////////////////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;

namespace VSCoverageToEmma
{
	public sealed class VSConverterLoader : IEnumerable<string>
	{
		private readonly Dictionary<string, IVSCoverageConverterFactory> factories_;

		public VSConverterLoader(string folderPath)
		{
			factories_ =
				(from path in
					Directory.GetFiles(folderPath, "VSCoverageToEmma.V*Converter.dll")
				 let assembly = LoadFrom(path)
				 where assembly != null
				 from type in assembly.GetTypes()
				 where
					(type.IsClass == true) &&
					(type.IsAbstract == false) &&
					(typeof(IVSCoverageConverterFactory).IsAssignableFrom(type) == true) &&
					(type.GetConstructor(Type.EmptyTypes) != null)
				 let factory = CreateFactory(type)
				 where factory != null
				 select factory).
				ToDictionary(factory => factory.Name.ToUpper(), factory => factory);
		}

		private static Assembly LoadFrom(string path)
		{
			try
			{
				return Assembly.LoadFrom(path);
			}
			catch
			{
				return null;
			}
		}

		private static IVSCoverageConverterFactory CreateFactory(Type type)
		{
			try
			{
				return (IVSCoverageConverterFactory)Activator.CreateInstance(type);
			}
			catch
			{
				return null;
			}
		}

		public IVSCoverageConverter Create(string name, string binariesFolderPath, string symbolsFolderPath)
		{
			var factory = factories_[name.ToUpper()];
			return factory.CreateConverter(binariesFolderPath, symbolsFolderPath);
		}

		public IEnumerator<string> GetEnumerator()
		{
			return factories_.Keys.OrderBy(keyName => keyName).GetEnumerator();
		}

		IEnumerator IEnumerable.GetEnumerator()
		{
			return this.GetEnumerator();
		}
	}
}
