﻿////////////////////////////////////////////////////////////////////////////////////////////////////
//
// VSCoverageToEmma - Visual Studio Coverage file to Emma converter
// Copyright (c) Kouji Matsui, All rights reserved.
//
// * Redistribution and use in source and binary forms, with or without modification,
//   are permitted provided that the following conditions are met:
// * Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
// * Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
////////////////////////////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Xml;
using System.Xml.Xsl;

namespace VSCoverageToEmma
{
	public sealed class VSCoverageToEmmaConverter
	{
		private static readonly XslCompiledTransform xslt_ = new XslCompiledTransform();
		private static readonly XmlWriterSettings writerSettings_ = new XmlWriterSettings()
			{
				ConformanceLevel = ConformanceLevel.Auto,
				Encoding = Encoding.UTF8,
				Indent = true,
				IndentChars = "\t",
				NewLineChars = Environment.NewLine,
				OmitXmlDeclaration = false
			};

		static VSCoverageToEmmaConverter()
		{
			using (var sr = new StringReader(Properties.Resources.MSTestCoverageToEmma))
			{
				using (var xr = XmlReader.Create(sr))
				{
					xslt_.Load(xr, new XsltSettings(false, false), new XmlUrlResolver());
				}
			}
		}

		private readonly IVSCoverageConverter converter_;
		private readonly SynchronizationContext context_;

		public VSCoverageToEmmaConverter(IVSCoverageConverter converter)
			: this(converter, SynchronizationContext.Current)
		{
		}

		public VSCoverageToEmmaConverter(IVSCoverageConverter converter, SynchronizationContext context)
		{
			converter_ = converter;
			context_ = context;
		}

		public EventHandler<ConvertedEventArgs> Converted;

		private void OnConverted(string vsCoveragePath, string emmaPath, int methods)
		{
			var converted = this.Converted;
			if (converted != null)
			{
				if (context_ != null)
				{
					context_.Post(dummy => converted(this, new ConvertedEventArgs(vsCoveragePath, emmaPath, methods)), null);
				}
				else
				{
					converted(this, new ConvertedEventArgs(vsCoveragePath, emmaPath, methods));
				}
			}
		}

		private static string SanitizeFolderPath(string path)
		{
			return (path.EndsWith(@"\") == true) ? path : (path + @"\");
		}

		public int ConvertCoverage(string vsCoveragePath, string emmaPath)
		{
			var methods = 0;

			using (var ms = new MemoryStream())
			{
				methods = converter_.Convert(ms, vsCoveragePath);
				ms.Position = 0;

				using (var xmlNativeCoverage = XmlReader.Create(ms))
				{
					using (var stream = new FileStream(emmaPath, FileMode.Create, FileAccess.ReadWrite, FileShare.None))
					{
						var xw = XmlWriter.Create(stream, writerSettings_);
						xslt_.Transform(xmlNativeCoverage, xw);

						xw.Flush();
						stream.Close();
					}
				}
			}

			this.OnConverted(vsCoveragePath, emmaPath, methods);

			return methods;
		}

		public int ConvertCoverages(string vsCoveragesFolderPath, IEnumerable<string> vsCoverageRelativePaths, string emmaFolderPath)
		{
			var vsCoverageFolderSanitizedPath = SanitizeFolderPath(vsCoveragesFolderPath);
			var emmaFolderSanitizedPath = SanitizeFolderPath(emmaFolderPath);

			var count = 0;

			Parallel.ForEach(
				from vsCoverageRelativePath in
					vsCoverageRelativePaths
				let vsCoveragePath = Path.Combine(vsCoveragesFolderPath, vsCoverageRelativePath)
				let fileName = Path.GetFileNameWithoutExtension(vsCoverageRelativePath)
				let emmaPath = Path.Combine(emmaFolderSanitizedPath, Path.Combine(Path.GetDirectoryName(vsCoverageRelativePath), fileName)) + ".xml"
				select new
				{
					VSCoveragePath = vsCoveragePath,
					EmmaPath = emmaPath
				},
				entry =>
				{
					var result = this.ConvertCoverage(entry.VSCoveragePath, entry.EmmaPath);
					Interlocked.Add(ref count, result);
				});

			return count;
		}

		public int ConvertCoverages(string vsCoveragesFolderPath, string emmaFolderPath)
		{
			var vsCoverageFolderSanitizedPath = SanitizeFolderPath(vsCoveragesFolderPath);
			var emmaFolderSanitizedPath = SanitizeFolderPath(emmaFolderPath);

			return this.ConvertCoverages(
				vsCoverageFolderSanitizedPath,
				from path in ConverterHelper.TraverseFiles(vsCoverageFolderSanitizedPath, "*.coverage")
				select path.Substring(vsCoverageFolderSanitizedPath.Length),
				emmaFolderSanitizedPath);
		}
	}
}
