﻿////////////////////////////////////////////////////////////////////////////////////////////////////
//
// VSCoverageToEmma - Visual Studio Coverage file to Emma converter
// Copyright (c) Kouji Matsui, All rights reserved.
//
// * Redistribution and use in source and binary forms, with or without modification,
//   are permitted provided that the following conditions are met:
// * Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
// * Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
////////////////////////////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Reflection;

namespace VSCoverageToEmma.Converters
{
	internal sealed class CoverageInfoAccessor
	{
		private readonly Type type_;
		private readonly MethodInfo createFromFileMethod_;

		public CoverageInfoAccessor(Type type)
		{
			Debug.Assert(type != null);
			Debug.Assert(type.IsClass == true);
			Debug.Assert(type.FullName == "Microsoft.VisualStudio.Coverage.Analysis.CoverageInfo");

			type_ = type;

			createFromFileMethod_ = type_.GetMethod(
				"CreateFromFile",
				BindingFlags.Public | BindingFlags.Static,
				null,
				new[] { typeof(string), typeof(IEnumerable<string>), typeof(IEnumerable<string>) },
				null);
			Debug.Assert(createFromFileMethod_ != null);

			var assemblyNameElements = type_.Assembly.FullName.Split(',').Select(element => element.Trim()).ToArray();

			AppDomain.CurrentDomain.AssemblyResolve += (s, e) =>
			{
				var elements = e.Name.Split(',').Select(element => element.Trim()).ToList();

				var version = elements[1];
				var publicKeyToken = elements[3];

				if ((version == assemblyNameElements[1]) && (publicKeyToken == assemblyNameElements[3]))
				{
					var assemblyName = elements[0];
					var path = Path.Combine(Path.GetDirectoryName(type_.Assembly.Location), assemblyName + ".dll");

					try
					{
						var assembly = Assembly.LoadFrom(path);
						return assembly;
					}
					catch (Exception)
					{
					}
				}

				return null;
			};
		}

		public DataSet BuildDataSet(string binariesFolderPath, string symbolsFolderPath, string vsCoveragePath)
		{
			var binaryPaths = ConverterHelper.TraverseFiles(binariesFolderPath, "*.exe").
				Concat(ConverterHelper.TraverseFiles(binariesFolderPath, "*.dll"));

			var symbolPaths = ConverterHelper.TraverseFiles(symbolsFolderPath, "*.pdb");

			using (var coverageInfo = (IDisposable)createFromFileMethod_.
				Invoke(null, new object[] { vsCoveragePath, binaryPaths, symbolPaths }))
			{
				var coverageInfoType = coverageInfo.GetType();
				var buildDataSetMethod = coverageInfoType.GetMethod(
					"BuildDataSet",
					BindingFlags.Public | BindingFlags.Instance,
					null,
					Type.EmptyTypes,
					null);
				Debug.Assert(buildDataSetMethod != null);

				return (DataSet)buildDataSetMethod.Invoke(coverageInfo, new object[0]);
			}
		}
	}
}
